@@ -17,20 +17,19 @@
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2021
 
 import importlib
-import warnings
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from typing import List, Dict, Any
 
 
 def import_extras(module_list):
+    # type: (List[str]) -> Dict[str, Any]
     out = dict()
     for mod in module_list:
         out[mod] = None
         try:
-            with warnings.catch_warnings():
-                # TODO: remove when https://github.com/paramiko/paramiko/issues/2038 is fixed
-                warnings.filterwarnings('ignore', 'Blowfish has been deprecated', module='paramiko')
-                # TODO: deprecated python 2 and 3.6 too ...
-                warnings.filterwarnings('ignore', 'Python .* is no longer supported', module='paramiko')
-                out[mod] = importlib.import_module(mod)
+            out[mod] = importlib.import_module(mod)
         except ImportError:
             pass
     return out
