@@ -26,7 +26,7 @@
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Tomas Javurek <tomas.javurek@cern.ch>, 2019-2020
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2019
+# - James Perry <j.perry@epcc.ed.ac.uk>, 2019-2021
 # - Gabriele Fronze' <gfronze@cern.ch>, 2019
 # - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
@@ -39,15 +39,18 @@
 # - Anil Panta <47672624+panta-123@users.noreply.github.com>, 2021
 # - Ilija Vukotic <ivukotic@cern.ch>, 2021
 # - David Población Criado <david.poblacion.criado@cern.ch>, 2021
+# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 from __future__ import absolute_import, print_function
 
+import argparse
 import base64
 import datetime
 import errno
 import getpass
 import hashlib
 import io
+import itertools
 import json
 import logging
 import mmap
@@ -60,6 +63,7 @@ import tempfile
 import threading
 import time
 import zlib
+from collections import OrderedDict
 from enum import Enum
 from functools import partial
 from uuid import uuid4 as uuid
@@ -71,8 +75,10 @@ from six.moves import StringIO, zip_longest as izip_longest
 from six.moves.urllib.parse import urlparse, urlencode, quote, parse_qsl, urlunparse
 from six.moves.configparser import NoOptionError, NoSectionError
 
-from rucio.common.config import config_get
-from rucio.common.exception import MissingModuleException, InvalidType, InputValidationError, MetalinkJsonParsingError, RucioException
+from rucio.common.config import config_get, config_has_section
+from rucio.common.exception import MissingModuleException, InvalidType, InputValidationError, MetalinkJsonParsingError, RucioException, \
+    DuplicateCriteriaInDIDFilter, DIDFilterSyntaxError, InvalidAlgorithmName
+
 from rucio.common.extra import import_extras
 from rucio.common.types import InternalAccount, InternalScope
 
@@ -114,6 +120,26 @@ codes = {
 DATE_FORMAT = '%a, %d %b %Y %H:%M:%S UTC'
 
 
+def dids_as_dicts(did_list):
+    """
+    Converts list of DIDs to list of dictionaries
+    :param did_list: list of DIDs as either "scope:name" or {"scope":"scope", "name","name"}
+    :returns: list of dictionaries {"scope":"scope", "name","name"}
+    """
+    out = []
+    for did in did_list:
+        if isinstance(did, str):
+            scope, name = did.split(":", 1)
+            did = dict(scope=scope, name=name)
+        if isinstance(did, dict):
+            if not ("name" in did and "scope" in did):
+                raise ValueError("Scope or name missing in: %s" % (did,))
+        else:
+            raise ValueError("Can not convert item %s (%s) to a DID" % (did, type(did)))
+        out.append(did)
+    return out
+
+
 def build_url(url, path=None, params=None, doseq=False):
     """
     utitily function to build an url for requests to the rucio system.
@@ -222,6 +248,19 @@ def is_checksum_valid(checksum_name):
     return checksum_name in GLOBALLY_SUPPORTED_CHECKSUMS
 
 
+def set_preferred_checksum(checksum_name):
+    """
+    A simple function to check wether a checksum algorithm is supported.
+    Relies on GLOBALLY_SUPPORTED_CHECKSUMS to allow for expandability.
+
+    :param checksum_name: The name of the checksum to be verified.
+    :returns: True if checksum_name is in GLOBALLY_SUPPORTED_CHECKSUMS list, False otherwise.
+    """
+    if is_checksum_valid(checksum_name):
+        global PREFERRED_CHECKSUM
+        PREFERRED_CHECKSUM = checksum_name
+
+
 def set_checksum_value(file, checksum_names_list):
     for checksum_name in checksum_names_list:
         if checksum_name in file['metadata'].keys() and file['metadata'][checksum_name]:
@@ -232,9 +271,7 @@ def set_checksum_value(file, checksum_names_list):
 
 def adler32(file):
     """
-    An Adler-32 checksum is obtained by calculating two 16-bit checksums A and B
-    and concatenating their bits into a 32-bit integer. A is the sum of all bytes in the
-    stream plus one, and B is the sum of the individual values of A from each step.
+    An Adler-32 checksum is obtained by calculating two 16-bit checksums A and B and concatenating their bits into a 32-bit integer. A is the sum of all bytes in the stream plus one, and B is the sum of the individual values of A from each step.
 
     :param file: file name
     :returns: Hexified string, padded to 8 values.
@@ -243,29 +280,15 @@ def adler32(file):
     # adler starting value is _not_ 0
     adler = 1
 
-    can_mmap = False
     try:
         with open(file, 'r+b') as f:
-            can_mmap = True
-    except:
-        pass
-
-    try:
-        # use mmap if possible
-        if can_mmap:
-            with open(file, 'r+b') as f:
-                m = mmap.mmap(f.fileno(), 0)
-                # partial block reads at slightly increased buffer sizes
-                for block in iter(partial(m.read, io.DEFAULT_BUFFER_SIZE * 8), b''):
-                    adler = zlib.adler32(block, adler)
-        else:
-            with open(file, 'rb') as f:
-                # partial block reads at slightly increased buffer sizes
-                for block in iter(partial(f.read, io.DEFAULT_BUFFER_SIZE * 8), b''):
-                    adler = zlib.adler32(block, adler)
-
+            # memory map the file
+            m = mmap.mmap(f.fileno(), 0)
+            # partial block reads at slightly increased buffer sizes
+            for block in iter(partial(m.read, io.DEFAULT_BUFFER_SIZE), b''):
+                adler = zlib.adler32(block, adler)
     except Exception as e:
-        raise Exception('FATAL - could not get Adler-32 checksum of file %s: %s' % (file, e))
+        raise Exception('FATAL - could not get Adler32 checksum of file %s - %s' % (file, e))
 
     # backflip on 32bit -- can be removed once everything is fully migrated to 64bit
     if adler < 0:
@@ -305,8 +328,8 @@ def sha256(file):
     :returns: string of 32 hexadecimal digits
     """
     with open(file, "rb") as f:
-        bytes = f.read()  # read entire file as bytes
-        readable_hash = hashlib.sha256(bytes).hexdigest()
+        bytes_ = f.read()  # read entire file as bytes
+        readable_hash = hashlib.sha256(bytes_).hexdigest()
         print(readable_hash)
         return readable_hash
 
@@ -389,10 +412,10 @@ def render_json(**data):
     return json.dumps(data, cls=APIEncoder)
 
 
-def render_json_list(list):
+def render_json_list(list_):
     """ JSON render function for list
     """
-    return json.dumps(list, cls=APIEncoder)
+    return json.dumps(list_, cls=APIEncoder)
 
 
 def datetime_parser(dct):
@@ -458,12 +481,21 @@ def grouper(iterable, n, fillvalue=None):
     return izip_longest(*args, fillvalue=fillvalue)
 
 
-def chunks(list, n):
+def chunks(list_, n):
     """
     Yield successive n-sized chunks from l.
     """
-    for i in range(0, len(list), n):
-        yield list[i:i + n]
+    for i in range(0, len(list_), n):
+        yield list_[i:i + n]
+
+
+def dict_chunks(dict_, n):
+    """
+    Iterate over the dictionary in groups of the requested size
+    """
+    it = iter(dict_)
+    for _ in range(0, len(dict_), n):
+        yield {k: dict_[k] for k in itertools.islice(it, n)}
 
 
 def my_key_generator(namespace, fn, **kw):
@@ -577,7 +609,16 @@ def _register_policy_package_surl_algorithms():
             package = config.config_get('policy', 'package' + ('' if not vo else '-' + vo['vo']))
             module = importlib.import_module(package)
             if hasattr(module, 'get_surl_algorithms'):
-                _SURL_ALGORITHMS.update(module.get_surl_algorithms())
+                surl_algorithms = module.get_surl_algorithms()
+                if not vo:
+                    _SURL_ALGORITHMS.update(surl_algorithms)
+                else:
+                    # check that the names are correctly prefixed
+                    for k in surl_algorithms.keys():
+                        if k.lower().startswith(vo['vo'].lower()):
+                            _SURL_ALGORITHMS[k] = surl_algorithms[k]
+                        else:
+                            raise InvalidAlgorithmName(k, vo['vo'])
         except (NoOptionError, NoSectionError, ImportError):
             pass
 
@@ -767,10 +808,10 @@ register_extract_scope_algorithm(extract_scope_atlas, 'atlas')
 register_extract_scope_algorithm(extract_scope_belleii, 'belleii')
 
 
-def extract_scope(did, scopes=None):
+def extract_scope(did, scopes=None, default_extract=_DEFAULT_EXTRACT):
     extract_scope_convention = config_get('common', 'extract_scope', False, None)
     if extract_scope_convention is None or extract_scope_convention not in _EXTRACT_SCOPE_ALGORITHMS:
-        extract_scope_convention = _DEFAULT_EXTRACT
+        extract_scope_convention = default_extract
     return _EXTRACT_SCOPE_ALGORITHMS[extract_scope_convention](did=did, scopes=scopes)
 
 
@@ -1051,7 +1092,7 @@ def parse_did_filter_from_string(input_string):
     :return: filter dictionary and type as string.
     """
     filters = {}
-    type = 'collection'
+    type_ = 'collection'
     if input_string:
         filter_options = input_string.replace(' ', '').split(',')
         for option in filter_options:
@@ -1081,7 +1122,7 @@ def parse_did_filter_from_string(input_string):
 
             if key == 'type':
                 if value.upper() in ['ALL', 'COLLECTION', 'CONTAINER', 'DATASET', 'FILE']:
-                    type = value.lower()
+                    type_ = value.lower()
                 else:
                     raise InvalidType('{0} is not a valid type. Valid types are {1}'.format(value, ['ALL', 'COLLECTION', 'CONTAINER', 'DATASET', 'FILE']))
             elif key in ('length.gt', 'length.lt', 'length.gte', 'length.lte', 'length'):
@@ -1100,6 +1141,114 @@ def parse_did_filter_from_string(input_string):
             else:
                 filters[key] = value
 
+    return filters, type_
+
+
+def parse_did_filter_from_string_fe(input_string, name='*', type='collection', omit_name=False):
+    """
+    Parse DID filter string for the filter engine (fe).
+
+    Should adhere to the following conventions:
+    - ';' represents the logical OR operator
+    - ',' represents the logical AND operator
+    - all operators belong to set of (<=, >=, ==, !=, >, <, =)
+    - there should be no duplicate key+operator criteria.
+
+    One sided and compound inequalities are supported.
+
+    Sanity checking of input is left to the filter engine.
+
+    :param input_string: String containing the filter options.
+    :param name: DID name.
+    :param type: The type of the did: all(container, dataset, file), collection(dataset or container), dataset, container.
+    :param omit_name: omit addition of name to filters.
+    :return: list of dictionaries with each dictionary as a separate OR expression.
+    """
+    # lookup table unifying all comprehended operators to a nominal suffix.
+    # note that the order matters as the regex engine is eager, e.g. don't want to evaluate '<=' as '<' and '='.
+    operators_suffix_LUT = OrderedDict({
+        '<=': 'lte',
+        '>=': 'gte',
+        '==': '',
+        '!=': 'ne',
+        '>': 'gt',
+        '<': 'lt',
+        '=': ''
+    })
+
+    # lookup table mapping operator opposites, used to reverse compound inequalities.
+    operator_opposites_LUT = {
+        'lt': 'gt',
+        'lte': 'gte'
+    }
+    operator_opposites_LUT.update({op2: op1 for op1, op2 in operator_opposites_LUT.items()})
+
+    filters = []
+    if input_string:
+        or_groups = list(filter(None, input_string.split(';')))     # split <input_string> into OR clauses
+        for or_group in or_groups:
+            or_group = or_group.strip()
+            and_groups = list(filter(None, or_group.split(',')))    # split <or_group> into AND clauses
+            and_group_filters = {}
+            for and_group in and_groups:
+                and_group = and_group.strip()
+                # tokenise this AND clause using operators as delimiters.
+                tokenisation_regex = "({})".format('|'.join(operators_suffix_LUT.keys()))
+                and_group_split_by_operator = list(filter(None, re.split(tokenisation_regex, and_group)))
+                if len(and_group_split_by_operator) == 3:       # this is a one-sided inequality or expression
+                    key, operator, value = [token.strip() for token in and_group_split_by_operator]
+
+                    # substitute input operator with the nominal operator defined by the LUT, <operators_suffix_LUT>.
+                    operator_mapped = operators_suffix_LUT.get(operator)
+
+                    filter_key_full = key
+                    if operator_mapped is not None:
+                        if operator_mapped:
+                            filter_key_full = "{}.{}".format(key, operator_mapped)
+                    else:
+                        raise DIDFilterSyntaxError("{} operator not understood.".format(operator_mapped))
+
+                    if filter_key_full in and_group_filters:
+                        raise DuplicateCriteriaInDIDFilter(filter_key_full)
+                    else:
+                        and_group_filters[filter_key_full] = value
+                elif len(and_group_split_by_operator) == 5:     # this is a compound inequality
+                    value1, operator1, key, operator2, value2 = [token.strip() for token in and_group_split_by_operator]
+
+                    # substitute input operator with the nominal operator defined by the LUT, <operators_suffix_LUT>.
+                    operator1_mapped = operator_opposites_LUT.get(operators_suffix_LUT.get(operator1))
+                    operator2_mapped = operators_suffix_LUT.get(operator2)
+
+                    filter_key1_full = filter_key2_full = key
+                    if operator1_mapped is not None and operator2_mapped is not None:
+                        if operator1_mapped:    # ignore '' operator (maps from equals)
+                            filter_key1_full = "{}.{}".format(key, operator1_mapped)
+                        if operator2_mapped:    # ignore '' operator (maps from equals)
+                            filter_key2_full = "{}.{}".format(key, operator2_mapped)
+                    else:
+                        raise DIDFilterSyntaxError("{} operator not understood.".format(operator_mapped))
+
+                    if filter_key1_full in and_group_filters:
+                        raise DuplicateCriteriaInDIDFilter(filter_key1_full)
+                    else:
+                        and_group_filters[filter_key1_full] = value1
+                    if filter_key2_full in and_group_filters:
+                        raise DuplicateCriteriaInDIDFilter(filter_key2_full)
+                    else:
+                        and_group_filters[filter_key2_full] = value2
+                else:
+                    raise DIDFilterSyntaxError(and_group)
+
+            # add name key to each AND clause if it hasn't already been populated from the filter and <omit_name> not set.
+            if not omit_name and 'name' not in and_group_filters:
+                and_group_filters['name'] = name
+
+            filters.append(and_group_filters)
+    else:
+        if not omit_name:
+            filters.append({
+                'name': name
+            })
     return filters, type
 
 
@@ -1330,7 +1479,8 @@ def setup_logger(module_name=None, logger_name=None, logger_level=None, verbose=
     '''
     # helper method for cfg check
     def _force_cfg_log_level(cfg_option):
-        cfg_forced_modules = config_get('logging', cfg_option, raise_exception=False, default=None, clean_cached=True)
+        cfg_forced_modules = config_get('logging', cfg_option, raise_exception=False, default=None, clean_cached=True,
+                                        check_config_table=False)
         if cfg_forced_modules:
             if re.match(str(cfg_forced_modules), module_name):
                 return True
@@ -1407,6 +1557,32 @@ def daemon_sleep(start_time, sleep_time, graceful_stop, logger=logging.log):
         graceful_stop.wait(sleep_time - time_diff)
 
 
+def is_client():
+    """"
+    Checks if the function is called from a client or from a server/daemon
+
+    :returns client_mode: True if is called from a client, False if it is called from a server/daemon
+    """
+    if 'RUCIO_CLIENT_MODE' not in os.environ:
+        try:
+            if config_has_section('database'):
+                client_mode = False
+            elif config_has_section('client'):
+                client_mode = True
+            else:
+                client_mode = False
+        except RuntimeError:
+            # If no configuration file is found the default value should be True
+            client_mode = True
+    else:
+        if os.environ['RUCIO_CLIENT_MODE']:
+            client_mode = True
+        else:
+            client_mode = False
+
+    return client_mode
+
+
 class retry:
     """Retry callable object with configuragle number of attempts"""
 
@@ -1435,3 +1611,73 @@ class retry:
                     logger(logging.DEBUG, str(e))
                 attempt -= 1
         return self.func(*self.args, **self.kwargs)
+
+
+class StoreAndDeprecateWarningAction(argparse.Action):
+    '''
+    StoreAndDeprecateWarningAction is a descendant of :class:`argparse.Action`
+    and represents a store action with a deprecated argument name.
+    '''
+
+    def __init__(self,
+                 option_strings,
+                 new_option_string,
+                 dest,
+                 **kwargs):
+        """
+        :param option_strings: all possible argument name strings
+        :param new_option_string: the new option string which replaces the old
+        :param dest: name of variable to store the value in
+        :param kwargs: everything else
+        """
+        super(StoreAndDeprecateWarningAction, self).__init__(
+            option_strings=option_strings,
+            dest=dest,
+            **kwargs)
+        assert new_option_string in option_strings
+        self.new_option_string = new_option_string
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        if option_string and option_string != self.new_option_string:
+            # The logger gets typically initialized after the argument parser
+            # to set the verbosity of the logger. Thus using simple print to console.
+            print("Warning: The commandline argument {} is deprecated! Please use {} in the future.".format(option_string, self.new_option_string))
+
+        setattr(namespace, self.dest, values)
+
+
+class StoreTrueAndDeprecateWarningAction(argparse._StoreConstAction):
+    '''
+    StoreAndDeprecateWarningAction is a descendant of :class:`argparse.Action`
+    and represents a store action with a deprecated argument name.
+    '''
+
+    def __init__(self,
+                 option_strings,
+                 new_option_string,
+                 dest,
+                 default=False,
+                 required=False,
+                 help=None):
+        """
+        :param option_strings: all possible argument name strings
+        :param new_option_string: the new option string which replaces the old
+        :param dest: name of variable to store the value in
+        :param kwargs: everything else
+        """
+        super(StoreTrueAndDeprecateWarningAction, self).__init__(
+            option_strings=option_strings,
+            dest=dest,
+            const=True,
+            default=default,
+            required=required,
+            help=help)
+        assert new_option_string in option_strings
+        self.new_option_string = new_option_string
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        super(StoreTrueAndDeprecateWarningAction, self).__call__(parser, namespace, values, option_string=option_string)
+        if option_string and option_string != self.new_option_string:
+            # The logger gets typically initialized after the argument parser
+            # to set the verbosity of the logger. Thus using simple print to console.
+            print("Warning: The commandline argument {} is deprecated! Please use {} in the future.".format(option_string, self.new_option_string))
